﻿//Vector3D.h:interface for the Vector3D class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __VECTOR3D_H__
#define __VECTOR3D_H__

#include <cmath>
#include <cassert>

#ifdef _MSC_VER
#pragma warning(disable : 4201) // anonymous unions were added in C11
#endif

#undef INLINE
#ifdef _MSC_VER
#include <cassert>
#define INLINE __forceinline
#elif defined(__GNUC__) || defined(__llvm__)
#undef assert
//#define assert(exp) TPS_DBG_ASSERT_EXPR(exp)
#define INLINE inline __attribute__((always_inline))
#endif

struct D3DXVECTOR3;

// Invert of sqrtf
INLINE float InvSqrt(float x)
{
	float xhalf = 0.5f * x;
	int i = reinterpret_cast<int &>(x);
	i = 0x5f3759df - (i >> 1);
	x = reinterpret_cast<float &>(i);
	x = x * (1.5f - xhalf * x * x);
	return x;
}

class Vector3D  // 3D vector
{
public:
	float x;
	float y;
	float z;
	INLINE static const Vector3D Zero()
	{
		return Vector3D(0.0f, 0.0f, 0.0f);
	}
	INLINE static const Vector3D Z()
	{
		return Vector3D(0.0f, 0.0f, 1.0f);
	}
	INLINE static const Vector3D Y()
	{
		return Vector3D(0.0f, 1.0f, 0.0f);
	}
	INLINE static const Vector3D X()
	{
		return Vector3D(1.0f, 0.0f, 0.0f);
	}
	INLINE Vector3D& operator = (const Vector3D& vector3D)
	{
		//assert(this != &vector3D && "Apropriation to itself");
		x = vector3D.x;
		y = vector3D.y;
		z = vector3D.z;
		return *this;
	}
	template<typename T>
	INLINE Vector3D& operator = (const T& data)
	{
		*this = data;
		return *this;
	}
	template<typename T>
	INLINE Vector3D& operator = (const T* data)
	{
		//assert(data && "NULL Pointer");
		//assert(sizeof(*this) <= sizeof(*data));

		*this = reinterpret_cast<const Vector3D &>(*data);
		return *this;
	}
	INLINE D3DXVECTOR3* D3D()
	{
		return reinterpret_cast<D3DXVECTOR3 *>(this);
	}
	INLINE const D3DXVECTOR3* D3D() const
	{
		return reinterpret_cast<const D3DXVECTOR3 *>(this);
	}
	INLINE operator float * ()
	{
		return &x;
	}
	INLINE operator const float * () const
	{
		return &x;
	}
	INLINE Vector3D operator - () const
	{
		return Vector3D(- x, - y, - z);
	}
	INLINE bool operator == (const Vector3D& vector) const	
	{
		return (x == vector.x && y == vector.y && z == vector.z);
	}
	INLINE bool operator != (const Vector3D& vector) const
	{
		return (x != vector.x || y != vector.y || z != vector.z);
	}
	INLINE Vector3D& operator += (const Vector3D& Vector)
	{
		x += Vector.x;
 		y += Vector.y;
 		z += Vector.z;
 		return *this;
	}
	INLINE Vector3D& operator -= (const Vector3D& Vector)
	{
		x -= Vector.x;
 		y -= Vector.y;
 		z -= Vector.z;
 		return *this;
	}
	INLINE Vector3D& operator *= (float Value)
	{
		x *= Value;
 		y *= Value;
 		z *= Value;
 		return *this;
	}
	INLINE Vector3D& operator /= (float Value)
	{
		//assert(Value && "Zero Value");
		*this *= (1.0f / Value);
		return *this;
	}
	INLINE Vector3D& operator += (float Value)
	{
		x += Value;
		y += Value;
		z += Value;
		return *this;
	}
	INLINE Vector3D& operator -= (float Value)
	{
		x -= Value;
		y -= Value;
		z -= Value;
		return *this;
	}		
	INLINE Vector3D& operator /= (const Vector3D& vector)
	{
		x /= vector.x;
 		y /= vector.y;
 		z /= vector.z;
 		return *this;
	}
	INLINE Vector3D& operator *= (const Vector3D& vector)
	{
		x *= vector.x;
 		y *= vector.y;
 		z *= vector.z;
 		return *this;
	}	
	// 
	INLINE float& operator [](int index)
	{
		//assert(index < 3 && "Out Of Range");
		return (&x)[index];
	}
	INLINE float operator [] (int index) const
	{
		//assert(index < 3 && "Out Of Range");
		return (&x)[index];
	}
	INLINE bool operator > (const Vector3D& v) const
	{
		return x > v.x && y > v.y && z > v.z;
	}
	INLINE bool operator >= (const Vector3D& v) const
	{
		return x >= v.x && y >= v.y && z >= v.z;
	}
	INLINE bool operator < (const Vector3D& v) const
	{
		return x < v.x && y < v.y && z < v.z;
	}
	INLINE bool operator <= (const Vector3D& v) const
	{
		return x <= v.x && y <= v.y && z <= v.z;
	}
	INLINE friend Vector3D operator + (const Vector3D& vector1, const Vector3D& vector2)
	{
		return Vector3D(vector1.x + vector2.x, vector1.y + vector2.y, vector1.z + vector2.z);
	}
	INLINE friend Vector3D operator + (const Vector3D& vector, float value)
	{
		return Vector3D(vector.x + value, vector.y + value, vector.z + value);
	}
	INLINE friend Vector3D operator - (const Vector3D& vector1, const Vector3D& vector2)
	{	
		return Vector3D(vector1.x - vector2.x, vector1.y - vector2.y, vector1.z - vector2.z);
	}		
	INLINE friend Vector3D operator - (const Vector3D& vector, float value)
	{
		return Vector3D(vector.x - value, vector.y - value, vector.z - value);
	}
	INLINE friend Vector3D operator * (const Vector3D& vector, float Value)
	{
		return Vector3D(vector.x * Value, vector.y * Value, vector.z * Value);
	}		
	INLINE friend Vector3D operator * (float Value, const Vector3D& vector)
	{
		return Vector3D(vector.x * Value, vector.y * Value, vector.z * Value);		
	}
	//
	INLINE friend Vector3D operator * (const Vector3D& v1, const Vector3D& v2)
	{
		return Vector3D(v1.x * v2.x, v1.y * v2.y, v1.z * v2.z);
	}
	INLINE friend Vector3D operator / (const Vector3D& v1, const Vector3D& v2)
	{
		//assert(v2.x && "Zero Value");
		//assert(v2.y && "Zero Value");
		//assert(v2.z && "Zero Value");
		return Vector3D(v1.x / v2.x, v1.y / v2.y, v1.z / v2.z);
	}
	INLINE friend Vector3D operator / (const Vector3D& vector, float Value)
	{
		//assert(Value && "Zero Value");
		Value = 1.0f / Value;
		return vector * Value;
	}
	INLINE friend Vector3D operator / (float Value, const Vector3D& vector)
	{
		return Vector3D ( Value / vector.x, Value / vector.y, Value / vector.z );
	}
	//
	INLINE float Length() const 
	{
		return sqrtf(LengthSq());
	}
	// 
	INLINE float LengthSq() const
	{
		return x * x + y * y + z * z;
	}
	INLINE float GetDistanceSqFrom(const Vector3D& other) const
	{
		Vector3D d = *this - other;
		float lenSq = d.LengthSq();
		return lenSq;
	}
	INLINE float GetDistanceFrom(const Vector3D& other) const
	{
		float lenSq = GetDistanceSqFrom(other);
		return sqrtf(lenSq);
	}
	INLINE float Get2DDistanceFrom(const Vector3D& other) const
	{
		float dx = x - other.x;
		float dz = z - other.z;
		float len = sqrtf(dx * dx + dz * dz);
		return len;
	}
	//
	INLINE Vector3D& Normalize()
	{	
		float l = LengthSq();
		//assert(l && "Zero Vector");
		l = InvSqrt(l);
		x *= l;
		y *= l;
		z *= l;
		return *this;
	}
	INLINE bool IsZero() const
	{
		return !x && !y && !z;
	}
	template<typename T>
	void Set(const T& X, const T& Y, const T& Z)
	{
		x = static_cast<float>(X);
		y = static_cast<float>(Y);
		z = static_cast<float>(Z);
	}
	INLINE Vector3D()
	{
	}
	template<typename T>
	INLINE Vector3D(T X, T Y, T Z) : x(static_cast<float>(X)), y(static_cast<float>(Y)), z(static_cast<float>(Z))
	{
	}
	INLINE Vector3D(float X, float Y, float Z) : x(X), y(Y), z(Z)
	{
	}
	INLINE Vector3D(const float* data) : x(data[0]), y(data[1]), z(data[2])
	{
		//assert(data && "NULL Pointer");
	}
	//template<typename T>
	//INLINE Vector3D(const T& data)
	//{
	//	assert(sizeof(Vector3D) >= sizeof(data) && "Invalid Data");
	//	*this = &data;
	//}
	//template<typename T>
	//INLINE Vector3D(const T* data)
	//{
	//	*this = data;
	//}
	INLINE Vector3D(const Vector3D& vector) : x(vector.x), y(vector.y), z(vector.z)
	{
	}
};

// dotProduct: a * b = |a| * |b| * cos(a, b)
INLINE float dotProduct(const Vector3D& vector1, const Vector3D& vector2)
{
	return vector1.x * vector2.x + vector1.y * vector2.y + vector1.z * vector2.z;
}

// dotProduct: a * b = |a| * |b| * cos(a, b)
INLINE float dotProduct(const float* vector1, const float* vector2)
{
	return dotProduct(Vector3D(vector1), Vector3D(vector2));
}

// a x b equal square are created from its vectors
INLINE Vector3D crossProduct(const Vector3D& v1, const Vector3D& v2)
{
	return Vector3D(v1.y * v2.z - v1.z * v2.y, -v1.x * v2.z + v1.z * v2.x, v1.x * v2.y - v1.y * v2.x);		
}

// a x b equal square are created from its vectors
INLINE Vector3D crossProduct(const Vector3D& v1, const float* v2)
{
	return crossProduct(v1, Vector3D(v2));
}

// a x b equal square are created from its vectors
INLINE Vector3D crossProduct(const float* v1, const float* v2)
{
	return crossProduct(Vector3D(v1), Vector3D(v2));
}

#endif // __VECTOR3D_H__

 
