﻿#ifndef __VERTEXFORMAT_H__
#define __VERTEXFORMAT_H__

#include "types.h"
#include "Vector2D.h"
#include "Vector3D.h"

struct VertexColor
{
	Vector3D pos;
	uint color;
	void Set(float x, float y, float z, uint Color)
	{
		pos.x = x;
		pos.y = y;
		pos.z = z;
		color = Color;
	}
	void Set(const Vector3D& v, uint Color)
	{
		Set(v.x, v.y, v.z, Color);
	}
	VertexColor& operator = (const Vector3D& v)
	{
		pos = v;
		return *this;
	}
	INLINE bool operator < (const VertexColor& v) const 
	{
		if (pos.x != v.pos.x) 
		{
			return pos.x < v.pos.x;
		}
		if (pos.y != v.pos.y) 
		{
			return pos.y < v.pos.y;
		}
		if (pos.z != v.pos.z) 
		{
			return pos.z < v.pos.z;
		}
		if (color != v.color) 
		{
			return color < v.color;
		}
		return false;
	}
	INLINE bool operator == (const VertexColor& v) const
	{
		return pos == v.pos && color == v.color;
	}
};

// textured vertex
struct TextureVertex
{
	// position
	Vector3D pos;
	struct uv
	{
		float x;
		float y;
		operator Vector2D& ()
		{
			return reinterpret_cast<Vector2D &>(*this);
		}
		void operator = (const Vector2D& v)
		{
			x = v.x;
			y = v.y;
		}
	};
	union
	{
		struct
		{
			// texcoord
			uv texCoord;
		};
		struct
		{
			float tu;
			float tv;
		};
	};
};

// Standart vertex(textured vertex with Normal)
struct Vertex
{
	// position
	Vector3D pos;
	//
	Vector3D normal;
	// texcoord
	Vector2D texCoord;
};

// Colored vertex with Normal
struct VertexNormalColor
{
	// position
	Vector3D pos;
	//
	Vector3D normal;
	// color
	uint color;
};

#endif //  __VERTEXFORMAT_H__
